<?xml version="1.0" encoding="UTF-8"?>
 
<#-- Import common macros and functions -->
<#import "macros_common_general.ftl" as com>
<#import "common_module_traversal_utils.ftl" as utils>
<#import "common_module_front_page_docbook.ftl" as fp>

<#assign locale = "en" />
<#assign sysDateTime = .now>

<#-- Initialize common variables -->
<@com.initializeMainVariables/>	

<#assign locale = "en" />
<#assign sysDateTime = .now>

<#-- Root entity -->
<#assign rootEntity = entity.root>

<#-- Define the main macro and the arguments to be used to get output using the traversal approach -->
<#assign mainMacroName = "addLiteratureReferencesForDocument"/>
<#assign mainMacroArgs = ['sectionDoc', 'sectionNode', 'entityDocument', 'level']/>

<#-- Make global the macroCall as well as each macro used in it -->
<#global mainMacroCall="<@" + mainMacroName + " " + mainMacroArgs?join(" ") + "/>"/>
<#global addLiteratureReferencesForDocument=addLiteratureReferencesForDocument/>

<#-- Initialize hash to store literature references information (input to table) -->
<#assign litRefsHash = {}/>

<book version="5.0" xmlns="http://docbook.org/ns/docbook" xmlns:xi="http://www.w3.org/2001/XInclude">

	<#-- Get front page -->
	<@fp.getFrontPage _subject _dossierHeader "Document L (List of Literature References) - sorted by Author"/>

	<#-- Chapters -->
	<chapter label="1">
		<title role="HEAD-1">List of Literature References (sorted by Author)</title>

		<#-- Do the traverse -->
		<@utils.traversal rootEntity 8/>
		
		<#-- Print output -->
		<@tableOutput litRefsHash/>

	</chapter>
</book>

<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->
<#macro addLiteratureReferencesForDocument sectionDoc sectionNode entityDocument level>

	<#if level==0>

		<#-- get the list of references: it's a list of lists, where the first element is the literature
		reference and the second the data protection claimed, which can be a picklist value, "N/A", or empty
		-->
  		<#assign listOfReferences = getReferencedLitRefEntitiesFromDoc(sectionDoc) >
    	<#if listOfReferences?has_content>
      		<#list listOfReferences as literatureReferenceDpList>
				<#local literatureReference=literatureReferenceDpList[0]/>
				<#local dataPr=literatureReferenceDpList[1]/>

				<#if literatureReference?has_content>

					<#-- Get the section name: Section + Endpoint -->
					<#local sectionName = "" />
					<#if sectionNode?has_content>
						<#if sectionNode.number?has_content>
							<#local sectionName = sectionNode.number + " ">
						</#if>
						<#local sectionName = sectionName + sectionNode.title>
					</#if>

					<#-- Endpoint: add to section name -->
					<#if sectionDoc.hasElement("AdministrativeData.Endpoint") && sectionDoc.AdministrativeData.Endpoint?has_content>
						<#local endpoint><@com.picklist picklistValue=sectionDoc.AdministrativeData.Endpoint printDescription=false printRemarks=false/></#local>
						<#local sectionName = sectionName + " (" + endpoint + ")" />
					</#if>

					<#-- Add to hash -->
					<#assign litRefsHash = addStudyToHash(litRefsHash, literatureReference, sectionDoc, sectionName, dataPr, entityDocument)/>
        		</#if>
      		</#list>
    	</#if>
    </#if>
</#macro>

<#-- This function retrieves all the literature references from a document -->
<#function getReferencedLitRefEntitiesFromDoc docElementNode>
	<#local listOfLiteratureReferences = [] />

	<#if docElementNode?node_type == "document_reference">
		<#local listOfLiteratureReferences = checkAndAddLiteraturReferenceWithDP(docElementNode, docElementNode, listOfLiteratureReferences) />
		<#return listOfLiteratureReferences />
	</#if>

	<#if docElementNode?node_type == "document_references">
		<#list docElementNode as item>
			<#local listOfLiteratureReferences = checkAndAddLiteraturReferenceWithDP(item, docElementNode, listOfLiteratureReferences) />
		</#list>
		<#return listOfLiteratureReferences />
	</#if>

	<#if docElementNode?children?has_content>
		<#list docElementNode?children as child>
			<#local listOfLiteratureReferences = listOfLiteratureReferences + getReferencedLitRefEntitiesFromDoc(child)  />
		</#list>
	</#if>
	<#return listOfLiteratureReferences/>
</#function>


<#-- This function is a new version of the one above, in order to include both the lit reference and data protection claim -->
<#function checkAndAddLiteraturReferenceWithDP docKey docNode listOfReferencedEntities>
	<#local doc = iuclid.getDocumentForKey(docKey) />
	<#if doc?has_content && doc.documentType == "LITERATURE">
		<#list listOfReferencedEntities as docDpList>
			<#local document=docDpList[0]/>
			<#if document.documentKey == doc.documentKey>
				<#return listOfReferencedEntities>
			</#if>
		</#list>

		<#-- get DP -->
		<#local dataProt=""/>
		<#local docKeyParents=docNode?ancestors/>
		<#if docKeyParents?has_content>
			<#local docKeyParent = docKeyParents[0]/>
			<#if docKeyParent.hasElement("DataProtectionClaimed")>
				<#local dataProt=docKeyParent.DataProtectionClaimed/>
			<#else>
				<#local dataProt="N/A"/>
			</#if>
		</#if>

		<#-- add -->
		<#local newLitEntry = [doc, dataProt]/>
		<#return listOfReferencedEntities + [newLitEntry]>
	</#if>
	<#return listOfReferencedEntities>
</#function>

<#-- This function appends a reference and a study into a pre-defined hashMap -->
<#function addStudyToHash hash reference document section datapr entity>

	<#local uuid = reference.documentKey.uuid/>
	<#local author><@com.text reference.GeneralInfo.Author/></#local>
	<#local hashKey = author + "-" + uuid/>

	<#-- if reference exists, add new document to its hash entry; else create a new entry-->
	<#if hash?keys?seq_contains(hashKey)>

		<#local hashEntry = hash[hashKey]/>
		<#local sects = hashEntry['sections']/>

		<#if sects?keys?seq_contains(section)>
			<#local docs = sects[section]['docs']/>
			<#local dps = sects[section]['dps']/>
			<#local ents = sects[section]['ents']/>

			<#local docs = docs + [document]/>
			<#local dps = dps + [datapr]/>
			<#local ents = ents + [entity]/>

			<#local sect = {section:{'docs':docs, 'dps': dps, 'ents': ents}}/>
		<#else>
			<#local sect = {section:{'docs': [document], 'dps' : [datapr], 'ents': [entity]}}/>
		</#if>

		<#local sects = sects + sect>

	<#else>

		<#local sects= {section: {'docs': [document], 'dps': [datapr], 'ents':[entity]}}/>

	</#if>

	<#local hashEntry = { hashKey : {'reference': reference, 'sections': sects}}/>
	<#local hash = hash + hashEntry/>

	<#return hash/>

</#function>

<#-- Table format for the literature references -->
<#macro tableOutput hash>
	<#-- final table output -->
	<para role="small">
	<table border="1">
        <title>Table of Literature References</title>
          
          <col width="8%" />
          <col width="4%" />
          <col width="28%" />
          <col width="6%"/>
          <col width="12%" />
          <col width="14%" />
          <col width="6%" />
          <col width="11%" />
          <col width="11%" />		
          		
          <tbody>
			<tr valign='middle' align='center'>		
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Author(s)</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Year</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Title<?linebreak?>
              																   Report No.<?linebreak?>
              																   Document No.<?linebreak?>
              																   Source, where different from company<?linebreak?>
              																   GLP<?linebreak?>
              																   Published or not</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Vert. study Y/N</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Owner</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">IUCLID section (endpoint)</emphasis></th>	       			
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Data protection Y/N</emphasis></th>	
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Justification if data protection is claimed</emphasis></th>	
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Previously used<?linebreak?> Y/N</emphasis></th>
            </tr>

			<#if hash?has_content>
				<#list hash?keys?sort as key>
					<#local value = hash[key]/>
					<#local literatureReference=value['reference']>
							
					<#-- variables to extract field labels -->
					<@iuclid.label for=literatureReference.GeneralInfo.Name var="title"/>
					<@iuclid.label for=literatureReference.GeneralInfo.ReportNo var="reportNo"/>
					<@iuclid.label for=literatureReference.GeneralInfo.CompanyOwnerStudyNo var="studyNo"/>
					<@iuclid.label for=literatureReference.GeneralInfo.Source var="source"/>
					<@iuclid.label for=literatureReference.GeneralInfo.CompanyOwner var="owner"/>
					<@iuclid.label for=literatureReference.GeneralInfo.TestLab var="testingFacility"/>
					<@iuclid.label for=literatureReference.GeneralInfo.LiteratureType var="refType"/>

					<#-- get refSpan and GLP across docs in all sections -->
					<#local GLPlist=[]/>
					<#local useRefSpan=true/>
					<#local refSpan=0/> 
					<#list value['sections'] as section, sectionHash>
						<#local refSpan = refSpan + sectionHash['docs']?size/> 
						
						<#-- get GLP -->
						<#list sectionHash['docs'] as doc>
							<#if doc.hasElement("MaterialsAndMethods.GLPComplianceStatement") && doc.MaterialsAndMethods.GLPComplianceStatement?has_content>
								<#local glp><@com.picklist picklistValue=doc.MaterialsAndMethods.GLPComplianceStatement printDescription=false printRemarks=false /></#local>
								<#if !GLPlist?seq_contains(glp)>
									<#local GLPlist = GLPlist + [glp]/>
								</#if>
							</#if>
						</#list>
					</#list>
					 
					<#-- iterate over sections -->
					<#list value['sections'] as sectionName, sectionHash>
						
						<#local docs = sectionHash['docs']/>
						<#local dps = sectionHash['dps']/>
						<#local ents = sectionHash['ents']/> 
		
						<#local sectSpan=docs?size/>
						<#local useSectSpan=true/>
						
						<#list docs as sectionDoc>
					 
							<#local dataProtection = dps[sectionDoc?index]/>
							<#local entityDoc = ents[sectionDoc?index]/> 

						 	<tr>      

							 	<#if useRefSpan>
							 	
									<#local refUrl=utils.getDocumentUrl(literatureReference) />

								    <#-- Author -->									
								    <td rowspan="${refSpan}">
								    	<#if literatureReference.GeneralInfo.Author?has_content>
						      				<@com.text literatureReference.GeneralInfo.Author />	
						      			<#else>
						          			<emphasis>No author provided</emphasis>
						      			</#if>
								    </td>
								  
								    <#-- Year -->				
								    <td rowspan="${refSpan}"> 
								      <#if literatureReference.GeneralInfo.ReferenceYear?has_content>
								      	<@com.number literatureReference.GeneralInfo.ReferenceYear />		
								      <#else>
								          <emphasis>No year provided</emphasis>
								      </#if>
								    </td>
								    
								    <#-- Title, report no, source, type -->
								    <#local sourceVal = literatureReference.GeneralInfo.Source />
								    <#local ownerVal = literatureReference.GeneralInfo.CompanyOwner />
								    <#local testingFacilityVal =literatureReference.GeneralInfo.TestLab/>
						    
								    <td rowspan="${refSpan}">   
								    	<ulink url="${refUrl}">
								      		<#if literatureReference.GeneralInfo.Name?has_content>
						          				<@com.text literatureReference.GeneralInfo.Name/> 	
					        				<#else>
						        				<emphasis><@com.text title/> not provided</emphasis>
						      				</#if>
								      	</ulink>
								    
								    	<para>
									      <#if literatureReference.GeneralInfo.ReportNo?has_content>
									          <@com.text literatureReference.GeneralInfo.ReportNo />
									      <#else>
									          <emphasis>Report No. not provided</emphasis>
									      </#if>
							  			</para>
							  			
							  			<para>
							      			<#if literatureReference.GeneralInfo.CompanyOwnerStudyNo?has_content>
							          			<@com.text literatureReference.GeneralInfo.CompanyOwnerStudyNo />
							        		<#else>
							          			<emphasis>Document No. not provided</emphasis>
							      			</#if>
						      			</para>
						      
									  	<para>
										  	<#if sourceVal?has_content || testingFacilityVal?has_content>
										  		<#if sourceVal!=ownerVal && sourceVal!=testingFacilityVal>
										        	<@com.text sourceVal/>	
										        </#if>
												
										        <#if testingFacilityVal!=ownerVal>
										        	<@com.text testingFacilityVal/>	
										        </#if>
									      	<#else>
									      		<emphasis>Source not provided</emphasis>  
										  	</#if>
									  	</para>
							  			
							  			<#-- GLP (needs to be across sections docs) -->
									  	<para>  
										  	<#if GLPlist?has_content>
										      	GLP: ${GLPlist?join("; ")}
										  	<#else>
										      	<emphasis>GLP information not available</emphasis>
										  	</#if>
									  	</para>
						      			
								      	<para>
									      	<#if refType?has_content>
									      		<#local litRefTypeVal><@com.picklist literatureReference.GeneralInfo.LiteratureType /></#local>
									      	
									      		<#if litRefTypeVal=="publication" || litRefTypeVal=="publication (copyright not owned for reproduction)" || litRefTypeVal=="review article or handbook">
									      			Published
									      		<#else>
									      			<#if sourceVal?has_content>
									      				Published
									      			<#else>
									      				Unpublished
									      			</#if>
									      		</#if>
									      		(${litRefTypeVal})
									      	<#else>
									          	<emphasis><@com.text refType/> not provided</emphasis>
									      	</#if>
								      	</para>
							  				  
								    </td>	
								  
								    <#-- Vertebrate study -->
						 			<td rowspan="${refSpan}">
						 				<#-- #OPTION1: leave blank. #OPTION2: Y for Tox, Ecotox, and residues () -->
						 			</td>
						 			
						 			<#-- Study sponsor -->				
								    <td rowspan="${refSpan}"> 
								     	<#if ownerVal?has_content>
								          <@com.text ownerVal/> 	
								        <#else>
								          <emphasis>Owner not provided</emphasis>
								      	</#if>
								    </td>
	
								    <#local useRefSpan=false/>
							    </#if>
							    
							    <#-- IUCLID Section name and number -->  
							    <#local sectDocUrl=utils.getDocumentUrl(sectionDoc)>
							    <td>
									<ulink url="${sectDocUrl}"><@com.text sectionName /></ulink>
								</td>
								 
								<#-- Data protection claimed yes or no -->
						    	<td>
							    	<#if dataProtection?has_content && !(dataProtection?is_string) >
							    		<#local dpValue><@com.picklist picklistValue=dataProtection printDescription=false printRemarks=false/></#local>	
						      			<#local dpValue=dpValue?replace("yes", "Y")/>
						      			<#local dpValue=dpValue?replace(", ", " (")/>
										${dpValue?replace("share", "share)")}
									<#elseif dataProtection?has_content>
										${dataProtection}
									<#else>
										N
							    	</#if>
							    </td>
							    
							    <td>
							    	<#if dataProtection?has_content && !(dataProtection?is_string) >
							    		<@com.text dataProtection.remarks/>
									<#else>
										${dataProtection}
							    	</#if>
					    		</td>
					    		
					    		<#-- Previously used (Change Log) -->
							    <td>     
								    <#local changeLogStatus=getChangeLogStatus(sectionDoc, entityDoc)/>
									${changeLogStatus} 
								</td>

							</tr>
						</#list>
					</#list>
				</#list>
			</#if>
		</tbody>
	</table>
	</para>
</#macro>

<#-- This function checks if the document exists in the change log and retrieves the status -->
<#function getChangeLogStatus study entity>
	<#local changeLogFlag="N"/>

	<#local changeLogEntryList=[]/>

	<#local changeLogs = iuclid.getSectionDocumentsForParentKey(entity.documentKey, "FLEXIBLE_RECORD", "ChangeLog") />
	<#list changeLogs as changeLog>
		<#list changeLog.ChangeLog.ChangeLogEntries as changeLogEntry>
			<#local changeLogDoc=iuclid.getDocumentForKey(changeLogEntry.LinkToDocument)/>

			<#if changeLogDoc?has_content>
				<#if study.documentKey.uuid==changeLogDoc.documentKey.uuid>
					<#local changeLogStatus><@com.picklist changeLogEntry.Status/></#local>
					<#if changeLogStatus?has_content>
						<#if !(changeLogStatus?starts_with("new"))>
							<#local changeLogFlag="Y"/>
						</#if>
						<#-- <#local changeLogStatusList = changeLogStatusList + [changeLogStatus]/> -->
						<#local changeLogEntryList = changeLogEntryList + [changeLogEntry]/>
					</#if>
				</#if>
			</#if>
		</#list>
	</#list>

	<#local changeLogStatusMessage>${changeLogFlag}
		<#if changeLogEntryList?has_content>
			<#list changeLogEntryList as clEntry>
				(<@com.picklist clEntry.Status/>)<#if clEntry.Remark?has_content> - <@com.text clEntry.Remark/></#if>
				<#if clEntry?has_next>|</#if>
			</#list>
		</#if>
	</#local>

	<#return changeLogStatusMessage>
</#function>